@@ -1,7 +1,7 @@
 /*
  *  SSL/TLS stress testing program
  *
- *  Copyright (C) 2006-2011, Brainspark B.V.
+ *  Copyright (C) 2006-2013, Brainspark B.V.
  *
  *  This file is part of PolarSSL (http://www.polarssl.org)
  *  Lead Maintainer: Paul Bakker <polarssl_maintainer at polarssl.org>
@@ -23,22 +23,43 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#ifndef _CRT_SECURE_NO_DEPRECATE
-#define _CRT_SECURE_NO_DEPRECATE 1
+#if !defined(POLARSSL_CONFIG_FILE)
+#include "polarssl/config.h"
+#else
+#include POLARSSL_CONFIG_FILE
 #endif
 
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
 
-#include "polarssl/config.h"
-
 #include "polarssl/net.h"
 #include "polarssl/ssl.h"
 #include "polarssl/entropy.h"
 #include "polarssl/ctr_drbg.h"
-#include "polarssl/timing.h"
 #include "polarssl/certs.h"
+#if defined(POLARSSL_TIMING_C)
+#include "polarssl/timing.h"
+#endif
+
+#if !defined(POLARSSL_BIGNUM_C) || !defined(POLARSSL_ENTROPY_C) ||  \
+    !defined(POLARSSL_SSL_TLS_C) || !defined(POLARSSL_SSL_SRV_C) || \
+    !defined(POLARSSL_SSL_CLI_C) || !defined(POLARSSL_NET_C) ||     \
+    !defined(POLARSSL_RSA_C) || !defined(POLARSSL_CTR_DRBG_C) ||    \
+    !defined(POLARSSL_X509_CRT_PARSE_C)
+int main( int argc, char *argv[] )
+{
+    ((void) argc);
+    ((void) argv);
+
+    printf("POLARSSL_BIGNUM_C and/or POLARSSL_ENTROPY_C and/or "
+           "POLARSSL_SSL_TLS_C and/or POLARSSL_SSL_SRV_C and/or "
+           "POLARSSL_SSL_CLI_C and/or POLARSSL_NET_C and/or "
+           "POLARSSL_RSA_C and/or POLARSSL_CTR_DRBG_C and/or "
+           "POLARSSL_X509_CRT_PARSE_C not defined.\n");
+    return( 0 );
+}
+#else
 
 #define OPMODE_NONE             0
 #define OPMODE_CLIENT           1
@@ -65,16 +86,6 @@
 #define DFL_SESSION_LIFETIME    86400
 #define DFL_FORCE_CIPHER        0
 
-/*
- * server-specific data
- */
-char *dhm_G = "4";
-char *dhm_P = 
-"E4004C1F94182000103D883A448B3F802CE4B44A83301270002C20D0321CFD00" \
-"11CCEF784C26A400F43DFB901BCA7538F2C6B176001CF5A0FD16D2C48B1D0C1C" \
-"F6AC8E1DA6BCC3B4E1F96B0564965300FFA1D0B601EB2800F489AA512C4B248C" \
-"01F76949A60BB7F00A40B1EAB64BDD48E8A700D60B7F1200FA8E77B0A979DABF";
-
 int server_fd = -1;
 
 /*
@@ -84,13 +95,15 @@ struct options
 {
     int opmode;                 /* operation mode (client or server)    */
     int iomode;                 /* I/O mode (blocking or non-blocking)  */
-    char *server_name;          /* hostname of the server (client only) */
+    const char *server_name;    /* hostname of the server (client only) */
     int server_port;            /* port on which the ssl service runs   */
     int command;                /* what to do: read or write operation  */
     int buffer_size;            /* size of the send/receive buffer      */
     int max_bytes;              /* max. # of bytes before a reconnect   */
     int debug_level;            /* level of debugging                   */
+#if defined(POLARSSL_TIMING_C)
     int conn_timeout;           /* max. delay before a reconnect        */
+#endif
     int max_connections;        /* max. number of reconnections         */
     int session_reuse;          /* flag to reuse the keying material    */
     int session_lifetime;       /* if reached, session data is expired  */
@@ -101,7 +114,7 @@ struct options
  * Although this PRNG has good statistical properties (eg. passes
  * DIEHARD), it is not cryptographically secure.
  */
-unsigned long int lcppm5( unsigned long int *state )
+static unsigned long int lcppm5( unsigned long int *state )
 {
     unsigned long int u, v;
 
@@ -118,35 +131,19 @@ unsigned long int lcppm5( unsigned long int *state )
     return( u );
 }
 
-void my_debug( void *ctx, int level, const char *str )
+static void my_debug( void *ctx, int level, const char *str )
 {
     if( level < ((struct options *) ctx)->debug_level )
         fprintf( stderr, "%s", str );
 }
 
-#if !defined(POLARSSL_BIGNUM_C) || !defined(POLARSSL_ENTROPY_C) ||  \
-    !defined(POLARSSL_SSL_TLS_C) || !defined(POLARSSL_SSL_SRV_C) || \
-    !defined(POLARSSL_SSL_CLI_C) || !defined(POLARSSL_NET_C) ||     \
-    !defined(POLARSSL_RSA_C) || !defined(POLARSSL_CTR_DRBG_C)
-int main( int argc, char *argv[] )
-{
-    ((void) argc);
-    ((void) argv);
-
-    printf("POLARSSL_BIGNUM_C and/or POLARSSL_ENTROPY_C and/or "
-           "POLARSSL_SSL_TLS_C and/or POLARSSL_SSL_SRV_C and/or "
-           "POLARSSL_SSL_CLI_C and/or POLARSSL_NET_C and/or "
-           "POLARSSL_RSA_C and/or POLARSSL_CTR_DRBG_C not defined.\n");
-    return( 0 );
-}
-#else
 /*
  * perform a single SSL connection
  */
 static int ssl_test( struct options *opt )
 {
     int ret, i;
-    int client_fd;
+    int client_fd = -1;
     int bytes_to_read;
     int bytes_to_write;
     int offset_to_read = 0;
@@ -161,32 +158,39 @@ static int ssl_test( struct options *opt )
     unsigned char *read_buf = NULL;
     unsigned char *write_buf = NULL;
 
-    char *pers = "ssl_test";
+    const char *pers = "ssl_test";
 
+#if defined(POLARSSL_TIMING_C)
     struct hr_time t;
+#endif
     entropy_context entropy;
     ctr_drbg_context ctr_drbg;
     ssl_context ssl;
-    x509_cert srvcert;
-    rsa_context rsa;
+    x509_crt srvcert;
+    pk_context pkey;
 
     ret = 1;
 
+    memset( &ssl, 0, sizeof(ssl_context) );
     entropy_init( &entropy );
+    x509_crt_init( &srvcert );
+    pk_init( &pkey );
+
     if( ( ret = ctr_drbg_init( &ctr_drbg, entropy_func, &entropy,
-                               (unsigned char *) pers, strlen( pers ) ) ) != 0 )
+                               (const unsigned char *) pers,
+                               strlen( pers ) ) ) != 0 )
     {
         printf( "  ! ctr_drbg_init returned %d\n", ret );
         goto exit;
     }
 
+#if defined(POLARSSL_TIMING_C)
     get_timer( &t, 1 );
+#endif
 
     memset( read_state, 0, sizeof( read_state ) );
     memset( write_state, 0, sizeof( write_state ) );
 
-    memset( &srvcert, 0, sizeof( x509_cert ) );
-    memset( &rsa, 0, sizeof( rsa_context ) );
 
     if( opt->opmode == OPMODE_CLIENT )
     {
@@ -200,7 +204,7 @@ static int ssl_test( struct options *opt )
         if( ( ret = ssl_init( &ssl ) ) != 0 )
         {
             printf( "  ! ssl_init returned %d\n\n", ret );
-            return( ret );
+            goto exit;
         }
 
         ssl_set_endpoint( &ssl, SSL_IS_CLIENT );
@@ -212,27 +216,27 @@ static int ssl_test( struct options *opt )
         printf("POLARSSL_CERTS_C not defined.\n");
         goto exit;
 #else
-        ret =  x509parse_crt( &srvcert, (unsigned char *) test_srv_crt,
-                              strlen( test_srv_crt ) );
+        ret =  x509_crt_parse( &srvcert, (const unsigned char *) test_srv_crt,
+                               strlen( test_srv_crt ) );
         if( ret != 0 )
         {
-            printf( "  !  x509parse_crt returned %d\n\n", ret );
+            printf( "  !  x509_crt_parse returned %d\n\n", ret );
             goto exit;
         }
 
-        ret =  x509parse_crt( &srvcert, (unsigned char *) test_ca_crt,
-                              strlen( test_ca_crt ) );
+        ret =  x509_crt_parse( &srvcert, (const unsigned char *) test_ca_list,
+                               strlen( test_ca_list ) );
         if( ret != 0 )
         {
-            printf( "  !  x509parse_crt returned %d\n\n", ret );
+            printf( "  !  x509_crt_parse returned %d\n\n", ret );
             goto exit;
         }
 
-        ret =  x509parse_key( &rsa, (unsigned char *) test_srv_key,
-                              strlen( test_srv_key ), NULL, 0 );
+        ret =  pk_parse_key( &pkey, (const unsigned char *) test_srv_key,
+                             strlen( test_srv_key ), NULL, 0 );
         if( ret != 0 )
         {
-            printf( "  !  x509parse_key returned %d\n\n", ret );
+            printf( "  !  pk_parse_key returned %d\n\n", ret );
             goto exit;
         }
 #endif
@@ -260,9 +264,12 @@ static int ssl_test( struct options *opt )
         }
 
         ssl_set_endpoint( &ssl, SSL_IS_SERVER );
-        ssl_set_dh_param( &ssl, dhm_P, dhm_G );
         ssl_set_ca_chain( &ssl, srvcert.next, NULL, NULL );
-        ssl_set_own_cert( &ssl, &srvcert, &rsa );
+        if( ( ret = ssl_set_own_cert( &ssl, &srvcert, &pkey ) ) != 0 )
+        {
+            printf( " failed\n  ! ssl_set_own_cert returned %d\n\n", ret );
+            goto exit;
+        }
     }
 
     ssl_set_authmode( &ssl, SSL_VERIFY_NONE );
@@ -272,12 +279,17 @@ static int ssl_test( struct options *opt )
     ssl_set_bio( &ssl, net_recv, &client_fd,
                        net_send, &client_fd );
 
-    if( opt->force_ciphersuite[0] == DFL_FORCE_CIPHER )
-          ssl_set_ciphersuites( &ssl, ssl_default_ciphersuites );
-    else  ssl_set_ciphersuites( &ssl, opt->force_ciphersuite );
+    if( opt->force_ciphersuite[0] != DFL_FORCE_CIPHER )
+        ssl_set_ciphersuites( &ssl, opt->force_ciphersuite );
 
     if( opt->iomode == IOMODE_NONBLOCK )
-        net_set_nonblock( client_fd );
+    {
+        if( ( ret = net_set_nonblock( client_fd ) ) != 0 )
+        {
+            printf( "  ! net_set_nonblock returned %d\n\n", ret );
+            return( ret );
+        }
+    }
 
      read_buf = (unsigned char *) malloc( opt->buffer_size );
     write_buf = (unsigned char *) malloc( opt->buffer_size );
@@ -333,7 +345,7 @@ static int ssl_test( struct options *opt )
 
         if( opt->command & COMMAND_READ )
         {
-            if( bytes_to_read == 0 )
+            while( bytes_to_read == 0 )
             {
                 bytes_to_read = rand() % opt->buffer_size;
                 offset_to_read = 0;
@@ -342,7 +354,7 @@ static int ssl_test( struct options *opt )
             ret = ssl_read( &ssl, read_buf + offset_to_read,
                             bytes_to_read );
 
-            if( ret >= 0 )
+            if( ret > 0 )
             {
                 for( i = 0; i < ret; i++ )
                 {
@@ -360,7 +372,9 @@ static int ssl_test( struct options *opt )
                 offset_to_read += ret;
             }
 
-            if( ret == POLARSSL_ERR_SSL_PEER_CLOSE_NOTIFY ||
+            if( ret == 0 ||
+                ret == POLARSSL_ERR_SSL_CONN_EOF ||
+                ret == POLARSSL_ERR_SSL_PEER_CLOSE_NOTIFY ||
                 ret == POLARSSL_ERR_NET_CONN_RESET )
             {
                 ret = 0;
@@ -382,9 +396,11 @@ static int ssl_test( struct options *opt )
               opt->max_bytes <= nb_written ) )
             break;
 
+#if defined(POLARSSL_TIMING_C)
         if( opt->conn_timeout != 0 &&
             opt->conn_timeout <= (int) get_timer( &t, 0 ) )
             break;
+#endif
     }
 
 exit:
@@ -398,14 +414,25 @@ exit:
         free( write_buf );
 
     ssl_close_notify( &ssl );
-    x509_free( &srvcert );
-    rsa_free( &rsa );
+    x509_crt_free( &srvcert );
+    pk_free( &pkey );
     ssl_free( &ssl );
-    net_close( client_fd );
+    ctr_drbg_free( &ctr_drbg );
+    entropy_free( &entropy );
+
+    if( client_fd != -1 )
+        net_close( client_fd );
 
     return( ret );
 }
 
+#if defined(POLARSSL_TIMING_C)
+#define USAGE_TIMING                                             \
+    "    conn_timeout=%%d (ms)        default: 0 (no timeout)\n"
+#else
+#define USAGE_TIMING ""
+#endif
+
 #define USAGE \
     "\n usage: ssl_test opmode=<> command=<>...\n"               \
     "\n acceptable parameters:\n"                                \
@@ -417,7 +444,7 @@ exit:
     "    buffer_size=%%d (bytes)      default: 1024\n"           \
     "    max_bytes=%%d (bytes)        default: 0 (no limit)\n"   \
     "    debug_level=%%d              default: 0 (disabled)\n"   \
-    "    conn_timeout=%%d (ms)        default: 0 (no timeout)\n" \
+    USAGE_TIMING                                                 \
     "    max_connections=%%d          default: 0 (no limit)\n"   \
     "    session_reuse=on/off        default: on (enabled)\n"    \
     "    session_lifetime=%%d (s)     default: 86400\n"          \
@@ -426,7 +453,7 @@ exit:
 
 int main( int argc, char *argv[] )
 {
-    int i, j, n;
+    int i;
     const int *list;
     int ret = 1;
     int nb_conn;
@@ -437,7 +464,7 @@ int main( int argc, char *argv[] )
     {
     usage:
         printf( USAGE );
-        
+
         list = ssl_list_ciphersuites();
         while( *list )
         {
@@ -456,7 +483,9 @@ int main( int argc, char *argv[] )
     opt.buffer_size             = DFL_BUFFER_SIZE;
     opt.max_bytes               = DFL_MAX_BYTES;
     opt.debug_level             = DFL_DEBUG_LEVEL;
+#if defined(POLARSSL_TIMING_C)
     opt.conn_timeout            = DFL_CONN_TIMEOUT;
+#endif
     opt.max_connections         = DFL_MAX_CONNECTIONS;
     opt.session_reuse           = DFL_SESSION_REUSE;
     opt.session_lifetime        = DFL_SESSION_LIFETIME;
@@ -464,14 +493,6 @@ int main( int argc, char *argv[] )
 
     for( i = 1; i < argc; i++ )
     {
-        n = strlen( argv[i] );
-
-        for( j = 0; j < n; j++ )
-        {
-            if( argv[i][j] >= 'A' && argv[i][j] <= 'Z' )
-                argv[i][j] |= 0x20;
-        }
-
         p = argv[i];
         if( ( q = strchr( p, '=' ) ) == NULL )
             continue;
@@ -535,10 +556,10 @@ int main( int argc, char *argv[] )
 
         if( strcmp( p, "debug_level" ) == 0 )
             opt.debug_level = atoi( q );
-
+#if defined(POLARSSL_TIMING_C)
         if( strcmp( p, "conn_timeout" ) == 0 )
             opt.conn_timeout = atoi( q );
-
+#endif
         if( strcmp( p, "max_connections" ) == 0 )
             opt.max_connections = atoi( q );
 
